      SUBROUTINE cv_diag_adiab(len,nd,ndp1,iflag_con,
     &                   iflag_clos,delt,
     &                   t1,q1,qs1,
     &                   p1,ph1,
     &                   sig1feed1,sig2feed1,wght1,
     &                   cape1,cin1,plcl1,plfc1,tvp1)
***************************************************************
*                                                             *
* CV_DRIVER                                                   *
*                                                             *
*                                                             *
* written by   : Sandrine Bony-Lena , 17/05/2003, 11.19.41    *
* modified by :                                               *
***************************************************************
*
C
      USE dimphy
      implicit none
C
C.............................START PROLOGUE............................
C
C PARAMETERS:
C      Name            Type         Usage            Description
C   ----------      ----------     -------  ----------------------------
C
C      len           Integer        Input        first (i) dimension
C      nd            Integer        Input        vertical (k) dimension
C      ndp1          Integer        Input        nd + 1
C      iflag_clos    Integer        Input        version of closure (0/1)
C      delt          Real           Input        time step
C      t1            Real           Input        temperature (sat draught envt)
C      q1            Real           Input        specific hum (sat draught envt)
C      qs1           Real           Input        sat specific hum (sat draught envt)
C      p1            Real           Input        full level pressure
C      ph1           Real           Input        half level pressure
C      sig1feed1     Real           Input        sigma coord at lower bound of feeding layer
C      sig2feed1     Real           Input        sigma coord at upper bound of feeding layer
C      wght1         Real           Input        weight density determining the feeding mixture
C      cape1         Real           Output       CAPE
C      cin1          Real           Output       CIN
C      tvp1          Real           Output       adiab lifted parcell virt temp
C S. Bony, Mar 2002:
C S. Bony, Oct 2002:
C	* Vectorization of convect3 (ie version lmd)
C
C..............................END PROLOGUE.............................
c
c
#include "dimensions.h"
c#include "dimphy.h"
c#include "cvparam3.h"
#include "cv3param.h"
c
c Input
      integer len
      integer nd
      integer ndp1
      integer iflag_con
      integer iflag_clos
      real delt
      real t1(len,nd)
      real q1(len,nd)
      real qs1(len,nd)
      real p1(len,nd)
      real ph1(len,ndp1)
      real sig1feed1 ! pressure at lower bound of feeding layer
      real sig2feed1 ! pressure at upper bound of feeding layer
      real wght1(nd) ! weight density determining the feeding mixture
c
c Output
      real cape1(len)
      real cin1(len)
      real plfc1(len)
      real plcl1(len)
      real tvp1(len,nd)
c
!-------------------------------------------------------------------
! --- ARGUMENTS
!-------------------------------------------------------------------
! --- On input:
!
!  t:   Array of absolute temperature (K) of dimension ND, with first
!       index corresponding to lowest model level. Note that this array
!       will be altered by the subroutine if dry convective adjustment
!       occurs and if IPBL is not equal to 0.
!
!  q:   Array of specific humidity (gm/gm) of dimension ND, with first
!       index corresponding to lowest model level. Must be defined
!       at same grid levels as T. Note that this array will be altered
!       if dry convective adjustment occurs and if IPBL is not equal to 0.
!
!  qs:  Array of saturation specific humidity of dimension ND, with first
!       index corresponding to lowest model level. Must be defined
!       at same grid levels as T. Note that this array will be altered
!       if dry convective adjustment occurs and if IPBL is not equal to 0.
!
!  p:   Array of pressure (mb) of dimension ND, with first
!       index corresponding to lowest model level. Must be defined
!       at same grid levels as T.
!
!  ph:  Array of pressure (mb) of dimension ND+1, with first index
!       corresponding to lowest level. These pressures are defined at
!       levels intermediate between those of P, T, Q and QS. The first
!       value of PH should be greater than (i.e. at a lower level than)
!       the first value of the array P.
!
!  nl:  The maximum number of levels to which convection can penetrate, plus 1.
!       NL MUST be less than or equal to ND-1.
!
!  delt: The model time step (sec) between calls to CONVECT
!
!----------------------------------------------------------------------------
! ---   On Output:
!
!
!
!-------------------------------------------------------------------
c
c  Local arrays
c

      integer i,k,n,il,j
      integer nword1,nword2,nword3,nword4
      integer icbmax
      integer iflag1(klon)
      integer nk1(klon)
      integer icb1(klon)
      integer icbs1(klon)

      logical ok_inhib  ! True => possible inhibition of convection by dryness

      real tnk1(klon)
      real thnk1(klon)
      real qnk1(klon)
      real gznk1(klon)
      real pnk1(klon)
      real qsnk1(klon)
      real unk1(klon)
      real vnk1(klon)
      real cpnk1(klon)
      real hnk1(klon)
      real pbase1(klon)
      real buoybase1(klon)

      real u1(klon,klev)
      real v1(klon,klev)
      real lv1(klon,klev)
      real cpn1(klon,klev)
      real tv1(klon,klev)
      real gz1(klon,klev)
      real hm1(klon,klev)
      real h1(klon,klev)
      real tp1(klon,klev)
      real clw1(klon,klev)
      real th1(klon,klev)
c
      integer ncum
c
      integer j1feed(klon)
      integer j2feed(klon)
      real p1feed1(len) ! pressure at lower bound of feeding layer
      real p2feed1(len) ! pressure at upper bound of feeding layer
      real wghti1(len,nd) ! weights of the feeding layers
c
c (local) compressed fields:
c
!      integer nloc
!      parameter (nloc=klon) ! pour l'instant

      integer idcum(klon)
      integer iflag(klon),nk(klon)
      integer nent(klon,klev)
      integer inb(klon), inbis(klon)

      real hp(klon,klev), ep(klon,klev), sigp(klon,klev)
      real buoy(klon,klev)
      real cape(klon)
      real cin(klon)
      real plfc(klon)
c
      real tnk(klon),qnk(klon),gznk(klon)
      real wghti(klon,nd)
      real hnk(klon),unk(klon),vnk(klon)
      real tvp(klon,klev) 
c
      real tvpbase, tvbase
c
!-------------------------------------------------------------------
! --- SET CONSTANTS AND PARAMETERS
!-------------------------------------------------------------------

c -- set simulation flags:
c   (common cvflag)

       CALL cv_flag

c -- set thermodynamical constants:
c 	(common cvthermo)

       CALL cv_thermo(3)

c -- set convect parameters
c
c 	includes microphysical parameters and parameters that
c  	control the rate of approach to quasi-equilibrium)
c 	(common cvparam)

       CALL cv3_param(nd,delt)

!---------------------------------------------------------------------
! --- INITIALIZE OUTPUT ARRAYS AND PARAMETERS
!---------------------------------------------------------------------
      DO k = 1,nd
      DO il = 1,len
       tvp1(il,k) = 0.
       tvp(il,k) = 0.
      ENDDO
      ENDDO
c
c
      DO k = 1,klev
      DO il = 1,klon
       u1(il,k) = 0.
       v1(il,k) = 0.
      ENDDO
      ENDDO
c
      DO il = 1,len
       cin1(il) = -100000.
       cape1(il) = -1.
       plfc(il) = -1.
      ENDDO
c
!--------------------------------------------------------------------
! --- CALCULATE ARRAYS OF GEOPOTENTIAL, HEAT CAPACITY & STATIC ENERGY
!--------------------------------------------------------------------

       CALL cv3_prelim(len,nd,ndp1,t1,q1,p1,ph1      ! nd->na
     o               ,lv1,cpn1,tv1,gz1,h1,hm1,th1)
c

!--------------------------------------------------------------------
! --- CONVECTIVE FEED
!--------------------------------------------------------------------
!
! compute feeding layer potential temperature and mixing ratio :
!
! get bounds of feeding layer
!
c test niveaux couche alimentation KE
       if(sig1feed1.eq.sig2feed1) then
               print*,'impossible de choisir sig1feed=sig2feed'
               print*,'changer la valeur de sig2feed dans physiq.def'
       stop
       endif
c
      if (iflag_clos .eq. 0 .or. iflag_con .eq.30) then
       do i=1,len
         p1feed1(i)=ph1(i,1)
         p2feed1(i)=ph1(i,2)
       end do
      else
       do i=1,len
         p1feed1(i)=sig1feed1*ph1(i,1)
         p2feed1(i)=sig2feed1*ph1(i,1)
       end do
      endif
!
!        do i=1,len
!          iflag1(i)=0
!        enddo
!initialisation par YU
       CALL cv3_feed(len,nd,t1,q1,u1,v1,p1,ph1,hm1,gz1           ! nd->na
     i         ,p1feed1,p2feed1,wght1
     o         ,wghti1,tnk1,thnk1,qnk1,qsnk1,unk1,vnk1
     o         ,cpnk1,hnk1,nk1,icb1,icbmax,iflag1,gznk1,plcl1)

c
!      print *, 'cv_diag_adiab,cv3_feed-> iflag1, plcl1 ',
!     $           iflag1(100),plcl1(100)
!      print*,'ph1,thnk1,icb1,icbmax,iflag1,gznk1,plcl1',
!     $           ph1(100,2),thnk1(100),icb1(100),icbmax,
!     $           iflag1(100),gznk1(100),plcl1(100)
!      print*,'cpnk1,hnk1,nk1,qnk1',
!     $        cpnk1(100),hnk1(100),nk1(100),qnk1(100)
!      print*,'len,nd,nk1,', len,nd,nk1
c
!--------------------------------------------------------------------
! --- UNDILUTE (ADIABATIC) UPDRAFT / 1st part
! (up through ICB for convect4, up through ICB+1 for convect3)
!     Calculates the lifted parcel virtual temperature at nk, the
!     actual temperature, and the adiabatic liquid water content.
!--------------------------------------------------------------------

       CALL cv3_undilute1(len,nd,t1,qs1,gz1,plcl1,p1,icb1,tnk1,qnk1  ! nd->na
     o                    ,gznk1,tp1,tvp1,clw1,icbs1)
   
!       print*,'qs1,tp1,icbs1',qs1(100,2),tp1(100,2),icbs1(100)
c
c
c ***   set cloud base buoyancy at (plcl+dpbase) level buoyancy
c
      do 100 i=1,len
       pbase1(i) = plcl1(i) + dpbase
       tvpbase = tvp1(i,icb1(i))*(pbase1(i)-p1(i,icb1(i)+1))
     :                        /(p1(i,icb1(i))-p1(i,icb1(i)+1))
     :         + tvp1(i,icb1(i)+1)*(p1(i,icb1(i))-pbase1(i))
     :                          /(p1(i,icb1(i))-p1(i,icb1(i)+1))
       tvbase = tv1(i,icb1(i))*(pbase1(i)-p1(i,icb1(i)+1))
     :                      /(p1(i,icb1(i))-p1(i,icb1(i)+1))
     :        + tv1(i,icb1(i)+1)*(p1(i,icb1(i))-pbase1(i))
     :                        /(p1(i,icb1(i))-p1(i,icb1(i)+1))
       buoybase1(i) = tvpbase - tvbase
100   continue
c
      ncum=len
c
!       print*,'dans cv_diag_adiab,len, ncum = ',len,ncum

!-------------------------------------------------------------------
! --- UNDILUTE (ADIABATIC) UPDRAFT / second part :
! ---   FIND THE REST OF THE LIFTED PARCEL TEMPERATURES
! ---   &
! ---   FIND THE LEVEL OF NEUTRAL BUOYANCY
!-------------------------------------------------------------------

       CALL cv3_undilute2(klon,ncum,nd,icb1,icbs1,nk1        !na->nd
     :                        ,tnk1,qnk1,gznk1,hnk1,t1,q1,qs1,gz1
     :                        ,p1,h1,tv1,lv1,pbase1,buoybase1,plcl1
     o                        ,inb,tp1,tvp1,clw1,hp,ep,sigp
     o                        ,buoy,iflag1)
!        print*,'tnk1,inb,buoy,iflag1',tnk1(100),inb(100),buoy(100,2),
!     $  iflag1(100)
c
!-------------------------------------------------------------------
! --- CLOSURE
!-------------------------------------------------------------------

c
        CALL cv_diag_cin(klon,ncum,nd,icb1,inb              ! na->nd
     :                       ,pbase1,plcl1,p1,ph1,tv1,tvp1,buoy
     o                       ,cape1,cin1,plfc1)

c
!      print *,'cv_diag_cin-> cape1 ',cape1(1)
!      print*,'pbase1,plcl1,ph1,cin1',pbase1(100),plcl1(100),ph1(100,2),
!     $        cin1(100)
c

9999  continue
      return
      end

