      subroutine mb_main(mlon,kmb,itap,dtime,mlev,
     $      u_e,v_e,p_e,paprs,t_e,
     $      tv_e,coef_T,theta_e,q_e,
     $      m_albsol,m_beta,cdragh,
     $      rlon,rlat,mu0,local_time,mlat_sun, swdn,lwdn,
     $      A_sol,B_sol,mfluxg1, mtsrf,
     $      mflux_swdn,mflux_sens,mflux_lat,mflux_lw,
     $      qsat_ts,qsat_t,qsat_e,
     $      eq_kk,buoy1,buoy2,buoy3,x1,x2,x3,
     $      m_wf,m_w,m_u,m_t,m_q,m_tv,m_buoy,m_top,
     $      m_icbs,m_plcl,gz_e_lcl,m_t_lcl,m_q_lcl,m_h_lcl,
     $      m_buoy_lcl,m_w_lcl,
     $      m_t_scd,m_tv_scd,m_buoy_scd,m_w_scd,
     $      ale_oro_therm,cin_oro_therm,m_trigger,m_trigger_pre,
     $      m_w_lfc,m_plfc,klfc,m_e_kine_lcl,
     $      m_p_sommet,m_p_sommet_moist,ztsol)
!
      USE dimphy
      IMPLICIT NONE
    

!########################
! Parametres fichi\'es    #
!########################
#include "dimensions.h"
*KEEP,dimphy.
CC#include "dimphy.h" 
*KEEP,YOMCST.
#include "YOMCST.h" 
*KEEP,YOETHF.
#include "YOETHF.h"
*KEEP,FCTTRE.
#include "FCTTRE.h"
#include "cvthermo.h"
#include "cv3param.h"
#include "conema3.h"
#include "iniprint.h"
*KEND.
!
!##################
!   Arguments     #
!##################
!
! mlon ------------------ input-R- number of grid cells with orography
! kmb(klon) ------------- input-R- kmb(i) = grid cell number of the i-th cell with orog.
! itap ------------------ input-R- time step number
! dtime ----------------- input-R- time step size
!
! eq_kk(klon,klev) -------output-R-
! buoy1(klon,klev) -------output-R-
! buoy2(klon,klev) -------output-R-
! buoy3(klon,klev) -------output-R-
! x1(klon,klev) ----------output-R-
! x2(klon,klev) ----------output-R-
! x3(klon,klev) ----------output-R-
!-----------------
! environnement
!-----------------
! u_e(klon,klev),v_e ---- input-R- vent de l'environnement (m/s)
! t_e(klon,klev) ---------input-R- temperature de l'environnement (K)
! tv_e(klon,klev) --------input-R- temperature virtuelle de l'environnement (K)
! rho_e(klon,klev) -------input-R- mass volumique de l'environnement (kg/m3)
! p_e(klon,klev)  --------input-R- pression de l'environnement au mileu
!                                  de la couche (Pa)
! paprs(klon,klev+1) -----input-R- pression pour chaque inter-couche (en Pa)
! coef_T(klon,klev) ------output-R- T/Theta
! theta_e(klon,klev) -----output-R- Potential temperature of environment
! q_e(klon,klev)  --------input-R- l'humidit\'e sp\'ecifique de l'environnement
!                                  au mileu de la couche (g/kg)?
! qsat_e(klon,klev) ------output-R- saturation specific humidity of environment
!
!------------
! mountain and sun
!------------
!
! mlev(klon) -------------input-I- nombre de niveau de relief
! rlon(klon) -------------input-R- longitude
! rlat(klon) -------------input-R- latitude
! mu0 --------------------input-R- cosine of the solar zenith angle (=0 at zenith)
! local_time -------------input-R-
! mlat_sun ---------------input-R- latitude du point subsolaire (rad)
!                                  \'egale "latsun" dans LMDZ4, subroutine  zenang
!
!--------------------
! brise de montagne 
!--------------------
!
! swdn(klon,klev+1) ------input-R- flux sw down (w/m2 )
! lwdn(klon,klev+1) ------input-R- flux lw down (w/m2 )
! A_sol(klon,klev), B_sol input-R- coef such that flux_sol = A_sol+B_sol*mtsrf
! mflux_swdn(klon,klev) --output-R- flux sw down net ( w/m2 )
! mflux_sens(klon,klev) --output-R- flux sensible \`a la surface de relief ( w/m2 )
! mflux_lat(klon,klev) ---output-R- flux latent \`a la surface de relief ( w/m2 )
! mflux_lw(klon,klev) ----output-R- flux longwave net \`a la surface de relief ( w/m2 )
!
! m_w(klon,klev) ---------output-R- vitesse verticale de brise (m/s)
! m_wf(klon,klev+1) ------output-R- vitesse verticale de brise \`a l'interface de couche
! m_u(klon,klev) ---------output-R- vitesse (horizontale) d'entrainement dans la brise
! m_t(klon,klev) ---------output-R- temp\'erature de brise (k)
! m_q(klon,klev) ---------output-R- humidit\'e sp\'ecifique de brise (kg/kg)
!
! m_plcl(klon)  ----------output-R- pression au niveau LCL
! m_t_lcl(klon)  ---------output-R- temp\'erature de brise au niveau LCL
! m_q_lcl(klon)  ---------output-R- humidit\'e de brise au niveau LCL
! gz_e_lcl(klon)  --------output-R- g\'eopotentielle au niveau LCL
! m_h_lcl(klon)  ---------output-I- \'energie static humide au niveau LCL
! m_w_lcl(klon) ----------output-R- w de brise au niveau de LCL
! m_buoy_lcl(klon) -------output-R- breeze buoyancy at LCL
! m_e_kine_lcl(klon) -----output-R- breeze kinetic energy at LCL
!
! m_plfc(klon)  ----------output-R- pression au niveau LFC
! m_w_lfc(klon) ----------output-R- w de brise au niveau de LFC
! klfc(klon) -------------output-R- grid level where free convection starts
!
! qsat_t(klon,klev) ------output-R- hidit\'e satur\'e \`a la temp\'erature de brise (kg/kg)
!
! m_icbs(klon)  ----------output-R- premier niveau endessous de LCL
! m_icb(klon)  -----------output-R- premier niveau endessus de LCL
! m_top(klon) ------------output-R- max niveau de mb s\`eche
! m_p_sommet(klon) -------output-R- pression of max niveau de mb complet
! m_p_sommet_moist(klon) -output-R-  pression of max niveau de mb moisture
!
! m_buoy(klon,klev) ------output-R- l'\'ecart temp\'erature entre la surface et
!                                   l'environnement(k)
! m_buoy2(klon,klev) -----output-R- buoyancy audessus de LCL
!
! ale_oro_therm(klon) ----output-R- energie de soul\`evement par l'effet
!                                   thermique de orographie (J/kg)
! cin_oro_therm(klon) ----output-R- inhibition convective dans brise (J/kg)
! m_trigger(klon) --------output-R- trigger for convection
! m_trigger_pre(klon) ----output-R- unused
!-------
! sol
!-------
!
! cdragh(klon) -----------input-R-  coefficient d'\'echange de chaleur \`a la surface
! m_beta(klon)  ----------input-R- evap relle/evap potential
! m_albsol(klon) ---------input-R- albedo de sol
! mtsrf(klon,klev) -------input/output-R- temp\'erature (sol) de la surface de relief (k)
! mfluxg1(klon,klev) -----output-R- flux conducitive thermique de premier couche de sol
! qsat_ts(klon,klev) -----output-R- humidit\'e satur\'e \`a la temp\'erature de sol (kg/kg)
!-----------------------------
!
!#####################
!Variables internes  #
!#####################
! slope(klon) -----------R- pente de relief
! msig(klev) ------------I- la fraction de surface de sol dans la boite
! dzh(klev) -------------R- hauteur de chaque couche(m)
! e_poten ---------------R- engergy potentiel 
! p_sens(klon,klev) -----R- l'\'ecart temp\'erature entre la surface et brise (k)
! m_theta(klon,klev) ----R- temp\'erature de brise (k)
! alp_oro_therm(klon) ---R- puissance de soul\`evement par mb (w/m2)         
!

!############################
!D\'eclaration des variables  #
!############################

!---------------------
!   Parameters 
!---------------------
!
       integer i,k,ip,itap,nloc,iflag_con,mlev(klon),mlon,kmb(klon),iq
       integer iflag_ale
       real dtime,Pi,DPMAX
!
!-------------------------------
!   l'air de l'environnement
!-------------------------------
!
        real p_e(klon,klev),paprs(klon,klev+1),
     $ p_e_coupl(klon,klev),paprs_coupl(klon,klev+1),
     $ theta_e(klon,klev),t_e(klon,klev),tv_e(klon,klev),q_e(klon,klev),
     $ rho_e(klon,klev),coef_T(klon,klev),
     $ gz_e(klon,klev),h_e(klon,klev),
     $ u_e(klon,klev),v_e(klon,klev),w_e(klon,klev)
      
!
!------------------------
!   g\'eom\'etrie de relief
!------------------------
!
        real slope_param,cdragh_param,mdx_param
        real slope(klon),dzh(klon,klev),msig(klev),dzh_2(klon,klev),mdx
!
!---------------------------------------
! caract\'eristiques de surface de relief
!---------------------------------------
!
        real m_albsol(klon),cdragh(klon),m_beta(klon)
!
!-------------------------------------------------------
! caract\'eristiques de g\'eographie et l'angle de soleille
!-------------------------------------------------------
!
        real rlat(klon), rlon(klon)
        real mlat_sun(klon),mu0(klon),psi_sl,local_time(klon)
!
!---------------------------------
! bilan de surface de relief
!---------------------------------
!
        real A_sol(klon,klev),B_sol(klon,klev),mtsrf(klon,klev),
     $  swdn(klon,klev),mflux_swdn(klon,klev),
     $  lwdn(klon,klev), mfluxg1(klon,klev),mflux_sens(klon,klev),
     $  mflux_lat(klon,klev),mflux_lw(klon,klev)
        real f_sens(klon,klev),f_lat(klon,klev)
 
!
!----------------------------------
! variables de breeze de montagne
!----------------------------------
    
        real eq_kk(klon,klev),buoy1(klon,klev),buoy2(klon,klev),
     $    buoy3(klon,klev),m_buoy(klon,klev)
        real x1(klon,klev), x2(klon,klev), x3(klon,klev)

        real wf2,m_wf(klon,klev+1),m_w(klon,klev),m_u(klon,klev),
     $    m_t(klon,klev), m_tv(klon,klev),m_q(klon,klev)
        integer m_top(klon)
        real m_p_sommet(klon),m_p_sommet_moist(klon)
        real dwfdTs(klon,klev),dTdTs(klon,klev),dqdTs(klon,klev)
        real  m_theta(klon,klev)

!---------------------------
! variables au niveaux LCL
!---------------------------
!
        real qsat_ts(klon,klev),qsat_e(klon,klev),qsat_t(klon,klev)
        real frac_lcl(klon),t_e_lcl(klon),tv_e_lcl(klon),m_plcl(klon),
     $    m_theta_lcl(klon),m_t_lcl(klon),m_tv_lcl(klon),
     $    m_q_lcl(klon),m_w_lcl(klon),gz_e_lcl(klon),m_h_lcl(klon),
     $    m_buoy_lcl(klon),m_lv(klon,klev),
     $    m_plcl_coupl (klon)
        integer  m_icbs(klon),m_icb(klon),m_nk(klon)
        real rh(klon),chi(klon)

        real m_q_lcl_test(klon), m_theta_lcl_test(klon), 
     $  m_w_lcl_test(klon), m_t_lcl_test(klon), m_tv_lcl_test(klon),
     $  m_buoy_lcl_test(klon),
     $  m_t2_test(klon,klev),m_tv2_test(klon,klev),
     $  m_buoy2_test(klon,klev), m_h_lcl_test(klon)
!
!-------------------------------------
! variables de brise sans condensation
!-------------------------------------
!
        
        real m_t_scd(klon,klev),m_tv_scd(klon,klev),
     $      m_buoy_scd(klon,klev),m_w_scd(klon,klev)
  
!
!---------------------------
! variables au niveaux LFC
!---------------------------
        real m_plfc(klon),m_w_lfc(klon),
     $       e_poten,kaka,frac_lfc,dzh_2_lfc,tv_e_lfc

!----------------------------------------------
! \'energies dans le syst\`eme de brise de montagne
!----------------------------------------------
!
        integer m_neg(klon)
        real ale_oro_therm(klon),cin_oro_therm(klon)
        integer kale(klon),kcin(klon),klfc(klon)
        logical ok_kale(klon),ok_kcin(klon),ok_klfc(klon),ok_plfc(klon)
        logical m_trigger(klon),m_trigger_pre(klon)
        real m_e_kine_lcl(klon)
!---------------------------------------------
! puissance de brise dans la maille
!---------------------------------------------
      
        real rho_ale(klon),m_surf(klon),alp_oro_therm(klon)
!
!------------------
!varibale test
         real test_m_wf1(klon,klev+1), test_m_wf1_bis(klon,klev+1),
     $        test_m_wf2(klon,klev+1) 
         real test_m_theta1(klon,klev+1),test_m_theta1_bis(klon,klev+1),
     $        test_m_theta2(klon,klev+1) 
         real test_m_q1(klon,klev+1), test_m_q1_bis(klon,klev+1),
     $        test_m_q2(klon,klev+1) 
         real gg_point(klon)
         real ztsol(klon)
!----------------------------------------------------
!
        Pi=2.*asin(1.)
!
!###########################################################################
!                                                                          #
! l'ordre d'appelle dans mb_main:                                          #
!-------------------------------------                                     #
!  m_swdn,m_env   : variables de l'environnement                           #
!                                                                          #
!  m_ini          : initiation des variables de brise                      #
!                                                                          #
!  m_qsat_ts      : calcul flux d'evaporation de la surface pour           #
!                m_breeze \`a partir de mtsrf de pas de temps pr\'ec\'edant      #  
!                                                                          #
!  m_breeze       : calucle m_tv endessous de LCL                          #
!                                                                          #
!  m_qsat_t,m_lcl : d\'eterminer LCL                                         #
!                                                                          #
!  m_undilute2    : calcule m_tv audessus de LCL                           #
!                                                                          #
!  m_cine         : calcule d'\'energy potentielle de brise \`a partir         #
!                    de profile de m_tv   
!                                                                          #
!  m_tsurf        : recalcule mtsrf \`a partir de m_w et m_t                 #
!###########################################################################
!
!####################################
!  Lecture de parametres
!####################################
!-------------------
! valeur par defaut
!-------------------
       cdragh_param=0.005
       slope_param = 18.
       iflag_ale = 0
       mdx_param = 100.

!------------------------------------------------
!lire les parametres pour le test de sensibilite
!-------------------------------------------------
!il faut donner la valeur chaque pas de temps, car ells se sauvegards pas
       open (99,file='mb_param.data',form='formatted',
     $         status='old',err=902)
         read(99,*,end=901) slope_param  ! slope = pi/slope_param
         read(99,*,end=901) mdx_param
         read(99,*,end=901) iflag_ale 
         read(99,*,end=901) cdragh_param ! cdrag impose si positif
!couplage avec le schema Emanuel
! iflag_ale = 0 -> ale_oro = 0.
!             1 -> ale_oro = m_e_kine_lcl
!             2 -> ale_oro = m_e_kine_lcl/(sin(slope))**2
!  
901   close (99)
902   continue
!------------------------------
       DO i = 1,klon
         slope(i) = Pi/slope_param
       ENDDO
       IF (cdragh_param .gt. 0.) THEN
        do i=1,klon
         cdragh(i)=cdragh_param
        enddo
       ENDIF
        mdx = mdx_param
!
       IF (itap .lt. 10) THEN
         print *,'-----------------------------------------------------'
         print *,'| Breeze: slope(1)= ',slope(1),' mdx= ',mdx,'  |'
         print *,'-----------------------------------------------------'
       ENDIF
!
!####################################
!  Flux swdn sur le pente de relief #                
!####################################
!
! input:   psi_sl,slope,rlat,rlon,mlat_sun,local_time,mu0,swdn,m_albsol
! output:  mflux_swdn
!
!-----------------------------------------------
c
      i=(klon/2)+1
      IF (prt_level .GE. 20) 
     $     print *, 'mb_main: lwdn ',(lwdn(i,k),k=1,klev)

! ---test pour mu0,swdn 
!      do iq=1,mlon
!           i=kmb(iq)
     
!          print*,'avant m_swdn','mu0',mu0(i),
!     $      'swdn',swdn(i,1)
!         enddo
!         print*,'avatn m_swdn'
!------------------------------------------
!jyg#####################################################################        
      IF (itap .GT. -1000) THEN    ! suppression calcul
!jyg#####################################################################        
        CALL m_swdn(mlon,kmb,slope,rlat,rlon,mlat_sun,
     $     local_time, mu0,swdn,m_albsol,mflux_swdn)

!-------------------------------------
!--- test pour mflux_swdn
!        do iq=1,mlon
!           i=kmb(iq)
!          print*,'apres m_swdn','slope',slope(i),
!     $         'mflux_swdn',mflux_swdn(i,1),mlat_sun,mlat_sun(i),
!     $         'mu0',mu0(i),'local_time',local_time(i),
!     $         'swdn',swdn(i,1)
!        enddo
!------------------------------------------------------
     
!
!###################
!  Environnement   #                  
!###################
!
! input:  u_e,v_e,p_e,paprs,t_e,q_e,slope
! output: dzh,msig,coef_T,theta_e,rho_e,tv_e,gz_e,h_e,w_e
!
         CALL m_env(mlon,kmb,u_e,v_e,p_e,paprs,t_e,q_e,slope,
     $        dzh,dzh_2,msig,mdx,coef_T,theta_e,rho_e,tv_e,gz_e,h_e,w_e)
!         do k=1,klev
!         print*,'1 coef_T',coef_T(1,k)
!         enddo
!
!################
!   Initiation  #               
!################
        CALL m_ini(mlon,kmb,p_e,paprs,theta_e,t_e,tv_e,q_e,
     $       p_e_coupl,paprs_coupl,
     $       mflux_sens,mflux_lat,mflux_lw,
     $       rh,chi,m_icbs,m_icb,
     $       t_e_lcl,tv_e_lcl,m_plcl,m_theta_lcl,m_t_lcl,m_tv_lcl,
     $       m_q_lcl,m_w_lcl,gz_e_lcl,m_h_lcl,m_buoy_lcl,m_lv,
     $       m_w_lfc,m_plfc,
     $       kale,kcin,klfc,ok_kale,ok_kcin,ok_klfc,ok_plfc,
     $       ale_oro_therm,cin_oro_therm,m_trigger)

!#############
!   qsat_ts  #               
!#############
!
! input:  mlev,p_e,mtsrf
! output:  qsat_ts
!

         CALL m_qsat_ts(mlon,kmb,mlev,p_e,mtsrf,qsat_ts)
!           print*,'itap,mtsrf apres m_qsat_ts',itap,mtsrf(100,1)
!#############
!   qsat_e   #               
!#############
!
! input:  mlev,p_e,t_e
! output:  qsat_e
!

         CALL m_qsat_e(mlon,kmb,mlev,p_e,t_e,qsat_e)
!
!######################
!  brise de montagne  #                   
!######################
!
! input:  mlev,cdragh,m_beta,slope,dzh,msig,mflux_swdn,lwdn,mtsrf,qsat_ts,
!         t_e,theta_e,tv_e,rho_e,q_e,coef_T,w_e
! output:  eq_kk,buoy1,buoy2,buoy3,m_buoy,
!          m_wf,m_w,m_theta,m_t,m_tv,m_q,
!          mfluxg1,mflux_sens,mflux_lat,mflux_lw,
!          m_top
!-------------------------------------------------
      CALL m_breeze1_wf_3e(rlat,rlon,mlon,kmb,itap,mlev,
     $        cdragh,m_beta,slope,dzh,dzh_2,msig,mdx,
     $        mtsrf,qsat_ts,f_sens,f_lat,
     $        t_e,theta_e,tv_e,q_e,qsat_e,coef_T,w_e,p_e,paprs,
     $        eq_kk,x1,x2,x3,m_buoy,
     $        m_wf,m_w,m_u,m_theta,m_t,m_tv,m_q,m_top,
     $        dwfdTs,dTdTs,dqdTs,ztsol)
!
1789  format ('    m_buoy     m_t        m_q          m_tv        ',
     $        't_e         q_e         tv_e ')
1790  format (7e12.4)
!      print *,'m_breeze1_wf_3e-> m_theta ',m_theta
!      print *,'dwfdTs ',dwfdTs
!      print *,'dTdTs ',dTdTs
!      print *,'dqdTs ',dqdTs
!---------------------------
!-- brise sans condensation
!Init
         do k=1,klev
           do iq=1,klon
                m_buoy_scd(i,k)=0.
                m_w_scd(i,k)=0.
                m_t_scd(i,k)=0.
                m_tv_scd(i,k)=0.
           enddo
         enddo
!
          do k=1,klev
           do iq=1,mlon
             i=kmb(iq)
                m_buoy_scd(i,k)=m_buoy(i,k)
                m_w_scd(i,k)=m_w(i,k)
                m_t_scd(i,k)=m_t(i,k)
                m_tv_scd(i,k)=m_tv(i,k)
           enddo
          enddo
!--------------------------------------------

!         do k=1,klev
!         print*,'2 coef_T',coef_T(1,k)
!         enddo
!      do k=1,klev
!      print*,'mb_main, m_u',m_u(1,k)
!      print*,'mb_main, m_w',m_w(1,k)
!      print*,'mb_main, m_buoy',m_buoy(1,k)
!      print*,'mb_main, m_theta',m_theta(1,k)
!      print*,'mb_main, m_w2',m_w2(1,k)
!      enddo
!--------------------------------------------------------------
!appelle m_tsurf avant m_breeze2, car m_wf et m_theta doit etre
!pres de surface
       CALL m_tsurf( mlon,kmb,itap,mlev,slope,dzh,msig,cdragh,m_beta,
     $  qsat_ts,A_sol,B_sol,
     $  theta_e,coef_T,q_e,rho_e,lwdn,mflux_swdn,
     $  m_wf,m_theta,m_q,
     $  mfluxg1,mflux_sens,mflux_lat,mflux_lw,mtsrf,
     $  dwfdTs,dTdTs,dqdTs) 
!      print*,'mb_main, m_beta',m_beta(1)
!      do k=1,klev
!      print*,'mb_main, m_wf',m_wf(1,k)
!      print*,'mb_main, m_theta',m_theta(1,k)
!      print*,'mb_main, mtsrf',mtsrf(1,k)
!      print*,'mb_main, dzh',dzh(1,k)
!      print*,'mb_main, swdn',swdn(1,k)<
!      print*,'mb_main, mflux_sens',mflux_sens(1,k)

!      print*,'mfluxg1,mflux_sens,mflux_lat,mflux_lw,swdn',
!     $mfluxg1(1,k),mflux_sens(1,k),mflux_lat(1,k),mflux_lw(1,k),
!     $swdn(1,k)
!      enddo
!-----------------------------------------------
!!
!#############
!   qsat_t   #               
!#############
!
! input:  mlev,p_e,m_t
! output:  qsat_t
!

       print *,' itap avant qsat_t ', itap
         CALL m_qsat_t(mlon,kmb,mlev,p_e,m_t,qsat_t)
!
!#########
!  LCL   #               
!######### 
!
! input: p_e,t_e,tv_e,q_e,gz_e,h_e,m_theta,m_t,m_tv,m_q,m_w,qsat_t,m_top 
! output: m_icbs,m_plcl,m_q_lcl,m_theta_lcl,m_t_lcl,m_tv_lcl,m_w_lcl,
!         gz_e_lcl,m_h_lcl,m_buoy_lcl
!    
         CALL m_lcl(itap,rlat,rlon,mlon,kmb,mlev,slope,
     $        p_e,t_e,tv_e,q_e,gz_e,h_e,coef_T,dzh_2,
     $        m_theta,m_t,m_tv,m_q,m_w,m_buoy,qsat_t,m_top,           
     $        rh,chi,m_icbs,m_plcl,
     $        m_q_lcl,m_theta_lcl,m_t_lcl,m_tv_lcl,
     $        m_w_lcl,gz_e_lcl,m_h_lcl,m_buoy_lcl,
     $  m_q_lcl_test,m_theta_lcl_test,m_t_lcl_test,m_tv_lcl_test,
     $  m_w_lcl_test,m_buoy_lcl_test)
!      print *,'m_lcl-> m_theta ',m_theta
!-----------------------------------

!#######################
!   Au dessus de LCL   #                 
!#######################

!- --- SET CONSTANTS AND PARAMETERS

      
c -- set thermodynamical constants:
c 	(common cvthermo)
       CALL cv_thermo(iflag_con)
c -- set convect parameters
c
c 	includes microphysical parameters and parameters that
c  	control the rate of approach to quasi-equilibrium)
c 	(common cvparam)
      if (iflag_con.eq.30) then
        CALL cv3_param(klev,dtime) ! nd:vertical dimension
      endif

         do iq=1,mlon
              i=kmb(iq)
          if (m_w_lcl(i).gt.0.) then
          m_icb(i) = m_icbs(i)+1
          m_nk(i) = m_icbs(i)
          m_plcl_coupl(i)=m_plcl(i)/100.!pour undilute2
          endif
! je consid\`ere que le niveau de cloud base est le niveau de lcl.
         enddo
!------------------------------------------------------------------
!      print *,'avant m_undilute3, m_icb,m_w_lcl,(m_w(i,k),k=1,10) ',
!     :                      m_icb,m_w_lcl,(m_w(i,k),k=1,10)
       CALL m_undilute3(itap,mlon,kmb,mlev,slope,
     :      m_icb,m_icbs,m_nk,      
     :      m_w_lcl,m_t_lcl,m_q_lcl,gz_e_lcl,m_h_lcl,
     :      t_e,q_e,qsat_e,gz_e,p_e_coupl,h_e,tv_e,m_lv,
     :      m_plcl_coupl,
     :      m_t,m_tv,m_buoy,m_w,dzh_2)
!      print *,'apres m_undilute3, m_icb,m_w_lcl,(m_w(i,k),k=1,10) ',
!     :                      m_icb,m_w_lcl,(m_w(i,k),k=1,10)
!
!attention:ce routine calcule les variables \`a partir de niveau m_icbs,
! il impose m_buoy2(i,m_icb(i)) = m_buoy_lcl.

!#################
! m_plfc
!#################
!d\'eterminer m_plfc,dans Emanuel scheme,plfc est le premi\`ere
!niveau o\`u la frottabilit\'e est positive,endessus de plcl.
!

! cas 1:inihibition de basse couche est non null,m_w2 depasse 
! son inertie.
       do k=2,klev
        do iq=1,mlon
            i= kmb(iq)
          if((m_w_lcl(i).gt.0.).and.(k.gt.m_icbs(i))) then
            if (.not.ok_plfc(i)) then
             if ((m_buoy(i,k).gt.0.).and.(m_buoy(i,k-1).lt.0.)) then
               if (m_w(i,k).gt.0.)then
!limiter la calcule de m_plfc entre k et k-1
!buoy(k)/(buoy(k)-buoy(k-1)=(plfc-p(k))/(p(k-1)-p(k))

            m_plfc(i)=p_e(i,k)+
     $                m_buoy(i,k)/(m_buoy(i,k)-m_buoy(i,k-1))*
     $                (p_e(i,k-1)-p_e(i,k))
!---- frac_lfc
!            frac_lfc = (m_plfc(i)-p_e(i,k))/(p_e(i,k-1)-p_e(i,k))
             frac_lfc= (p_e(i,k-1)-m_plfc(i))/(p_e(i,k-1)-p_e(i,k))
!
!---- tv_e_lfc
            tv_e_lfc =
! tv_e(i,k-1)+(tv_e(i,k)-tv_e(i,k-1))*frac_lfc
     $              frac_lfc*(tv_e(i,k)-tv_e(i,k-1))+tv_e(i,k-1)  
!---- dzh_2_lfc
!dzh:distance entre deux bord de couche
!               dzh(i,k) = (paprs(i,k)-paprs(i,k+1))/(rho_e(i,k)*RG)
!dzh_2:distance entre deux mileu de couche,k-1 et k
!               dzh_2(i,1) = 0.5*dzh(i,1)
!               dzh_2(i,k) =  0.5*dzh(i,k)+0.5*dzh(i,k-1)
!dzh_2_lfc:distance entre les niveaux (k-1) et lfc
            dzh_2_lfc = dzh_2(i,k)*frac_lfc
!trouver exitance des cas m_plfc>m_plcl
            ok_plfc(i)=.true.
!- interpolation nonlinear:
!evider le  probleme quand m_plcl et m_plfc est dans meme couche
         e_poten= RG*
     $     m_buoy(i,k-1)/(tv_e(i,k-1)+tv_e_lfc)*dzh_2_lfc       
!cape=RD*(Tp-Ta)dlnp
               kaka=e_poten+0.5*m_w(i,k-1)**2
                if (kaka.gt.0) then
                  m_w_lfc(i)=sqrt(2.*kaka)
                else
                  m_w_lfc(i)=0.
                  m_plfc(i)=0.
                  m_buoy(i,k)=0.
                  m_w(i,k)=0.
!m_buoy(k) et m_w(k) =0. quand kaka<0.(test pour kaka)
                endif! (kaka.gt.0)
             endif! (m_w(i,k).le.0.)
            endif! ((m_buoy(i,k).gt.0.).and.(m_buoy(i,k-1).le.0.))
           endif! (.not.ok_plfc(i))
          endif! (m_w_lcl(i).gt.0.).and.(k.gt.m_icbs(i)))
        enddo
       enddo
!---------------------------------------------------------
!-- control pour m_w apres m_plfc
          do k=1,klev-1
              do iq=1,mlon
                if ((k.gt.mlev(i)).and.(m_w(i,k-1).eq.0.))then
                  m_w(i,k)=0.
                  m_buoy(i,k)=0.
                endif
              enddo
          enddo 
!---------------------------------------- 
! cas 2:inihibition de basse couche est null,m_w2 depasse 
C m_plcl d'une pressure interval (=DPMAX) .
C
         DPMAX=5000.
        do k=1,klev
         do iq=1,mlon
            i= kmb(iq)
          if (m_w(i,k).gt.0.)then
            if (.not.ok_plfc(i)) then
!eviter de decouvrir le cas 1
             if (p_e(i,k).lt.(m_plcl(i)-DPMAX)) then
!               print*,'p_e,m_plcl',p_e(i,k),m_plcl(i)
             m_plfc(i)=m_plcl(i)
             m_w_lfc(i)=m_w_lcl(i)
            endif
           endif
          endif
         enddo
        enddo
!----------------------------------------------
!test le cas de m_plcl mais pas m_plfc
         do iq=1,mlon
            i= kmb(iq)
               gg_point(i)=0.
         enddo
    
!################
! m_w_lfc
!################

!-----------------------------------
! -- m_p_sommet,m_p_sommet_moist
!sommet de breeze determine par m_w2
       do i=1,klon
         m_p_sommet(i)=0.
         m_p_sommet_moist(i)=0.
       enddo

       do k=2,klev
        do iq=1,mlon
            i= kmb(iq)
             if ((m_w_scd(i,k).eq.0.).and.(m_w_scd(i,k-1).gt.0.)) then
                 m_p_sommet(i)=p_e(i,k)/100.
                 m_p_sommet_moist(i)=m_p_sommet(i)
             endif
             if (m_w_lcl(i).gt.0.) then
               if ((m_w(i,k).eq.0.).and.(m_w(i,k-1).gt.0.))  then
                 m_p_sommet_moist(i)=p_e(i,k)/100.
               endif
             endif
        enddo
       enddo
!

!#########################
! Energies et puissance  #                
!#########################
!
!----------------
! cin_oro_therm
!----------------
!--- input:
! * environement: p_e_coupl,paprs,tv_e
! * m_breeze: m_icb, m_pbase,m_plcl_coupl
! * m_undilute2: m_inb,m_tv2
!--- output:cina,cinb

!-----------------------------------------------
! m_e_kine_lcl
!-----------------------------------------------
            do i=1,klon
               m_e_kine_lcl(i)=0.
            enddo
            do iq=1,mlon
                i= kmb(iq)
                  m_e_kine_lcl(i)= 0.5*m_w_lcl(i)**2
            enddo

!   Determination de Ale_oro_therm
            do i=1,klon
               ale_oro_therm(i)=0.
            enddo
!
        IF ( iflag_ale .eq. 1) THEN
            do iq=1,mlon
                i= kmb(iq)
                  ale_oro_therm(i)= m_e_kine_lcl(i)
            enddo
        ENDIF
!

!-------------------------------------------------------             
! j'utilise plus m_cine pour la calcule de ale et cin, comme la condition de 
! d\'eclenchement est ale>>cin, le pr\'ecision n'est plus important 
!------------------------------            

!-------------------------------------------------------------------
!enfait, tous ca est equivalent d'une seule crit\`ere:
!m_buoy2 est positive au niveau just audessus de 800hpa.
!--------------------------------------------------------------------

!-------------------------------------------
!d\'eclenchement de DC(deep convection)
!-------------------------------------------
! d\'eclenchement de DC se traduire en m_w_lfc>0
! soit en therm d'\'energie cin\'etique:
! ale_oro_therm>cin_oro_therm     
!
     
       do iq=1,mlon
            i= kmb(iq)
             if (m_w_lfc(i).gt.0.) then
                m_trigger(i)=.true.
          endif
       enddo

!
!*********************************************************
!jyg#####################################################################        
      ENDIF    ! fin suppression
!jyg#####################################################################        
      return
      end


